#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
""" Capability endpoints """

import logging

import prometheus_client
from pyramid.config import Configurator
from pyramid.wsgi import wsgiapp

logger = logging.getLogger(__name__)


prometheus_exporter = prometheus_client.make_wsgi_app()


@wsgiapp
def metrics(environ, start_response):
    """
    Export metrics
    :param environ:
    :param start_response:
    :return:
    """
    return prometheus_exporter(environ, start_response)


def includeme(config: Configurator):
    """
    Function that gets included in server.py:main(); calls all route adding functions

    :param config: Pyramid server config object
    """
    default_routes(config)
    capability_routes(config)
    capability_request_routes(config)
    capability_version_routes(config)
    capability_execution_routes(config)
    casa_matrix_routes(config)


def default_routes(config: Configurator):
    """
    Uncategorized server routes

    :param config: Pyramid server config object
    """
    config.add_route("home", "/")
    config.add_view(metrics, name="metrics")
    config.add_route("metrics", "metrics")
    config.add_route(name="healthcheck", pattern="healthcheck")


def capability_routes(config: Configurator):
    """
    Server routes related to capabilities

    :param config: Pyramid server config object
    """
    capability_url = "capability/{capability_name}"
    # GET
    config.add_route(name="view_capability", pattern=f"{capability_url}", request_method="GET")
    config.add_route(name="view_active_capability_requests", pattern=f"{capability_url}/request")
    config.add_route(name="view_created_capability_requests", pattern=f"{capability_url}/created-requests")
    config.add_route(name="view_enabled_capabilities", pattern="/capabilities", request_method="GET")
    config.add_route(
        name="retrieve_available_qa_staff_in_group",
        pattern="/capabilities/available_qa_staff",
        request_method="GET",
    )
    config.add_route(name="get_analyst_email", pattern="capabilities/analyst_email", request_method="GET")

    # POST
    config.add_route(
        name="create_capability",
        pattern="capability/create",
        request_method="POST",
    )
    config.add_route(
        name="edit_capability",
        pattern=f"{capability_url}",
        request_method="POST",
    )
    config.add_route(name="enable_capability", pattern=f"{capability_url}/enable", request_method="POST")
    config.add_route(name="disable_capability", pattern=f"{capability_url}/disable", request_method="POST")
    config.add_route(name="pause_capability", pattern=f"{capability_url}/pause", request_method="POST")
    config.add_route(name="unpause_capability", pattern=f"{capability_url}/unpause", request_method="POST")


def capability_request_routes(config: Configurator):
    """
    Server routes related to capability requests

    :param config: Pyramid server config object
    """
    request_url = "capability/request/{request_id}"

    # GET
    config.add_route(name="view_capability_request", pattern=f"{request_url}", request_method="GET")

    # POST
    config.add_route(
        name="create_capability_request",
        pattern="capability/{capability_name}/request/create",
        request_method="POST",
    )
    config.add_route(
        name="create_and_submit_capability_request",
        pattern="capability/{capability_name}/request/create-and-submit",
        request_method="POST",
    )

    config.add_route(name="close_capability_request", pattern=f"{request_url}/close", request_method="POST")

    version_url = request_url + "/version/{version}"
    followon_pattern = f"{version_url}" + "/followon/{followon_type}"
    config.add_route(
        name="create_follow_on_capability_request",
        pattern=followon_pattern,
        request_method="POST",
    )
    config.add_route(
        name="edit_capability_request",
        pattern=f"{request_url}",
        request_method="POST",
    )
    config.add_route(name="cancel_capability_request", pattern=f"{request_url}/cancel", request_method="POST")
    config.add_route(
        name="assign_capability_request",
        pattern=f"{request_url}" + "/assign/group/{group}/staff/{user_name}",
        request_method="POST",
    )
    config.add_route(
        name="update_system_messages",
        pattern=f"{request_url}" + "/system_msg/{msg_id}/{action}",
        request_method="POST",
    )

    # DELETE
    config.add_route(name="delete_capability_request", pattern=f"{request_url}", request_method="DELETE")

    # Update the workflow metadata
    config.add_route(
        name="change_srdp_compatibility_version_metadata",
        pattern="capability/request/{capability_request_id}/srdp_compatible",
        request_method="POST",
    )


def capability_version_routes(config: Configurator):
    """
    Server routes related to capability versions

    :param config: Pyramid server config object
    """
    request_url = "capability/request/{capability_request_id}/version"
    # GET
    config.add_route(
        name="view_all_versions",
        pattern=f"{request_url}s",
        request_method="GET",
    )
    config.add_route(
        name="view_latest_version",
        pattern=f"{request_url}/latest",
        request_method="GET",
    )
    config.add_route(
        name="view_specific_version",
        pattern="capability/request/{capability_request_id}/version/{version_id}",
        request_method="GET",
    )
    config.add_route(
        name="view_internal_notes",
        pattern="capability/request/{capability_request_id}/version/{version_id}/internal_notes",
        request_method="GET",
    )

    # POST
    config.add_route(
        name="create_capability_version",
        pattern=f"{request_url}/create",
        request_method="POST",
    )
    config.add_route(
        name="submit_capability_version",
        pattern="capability/request/{capability_request_id}/version/{version_number}/submit",
        request_method="POST",
    )
    config.add_route(
        name="cancel_capability_version",
        pattern="capability/request/{capability_request_id}/version/{version_number}/cancel",
        request_method="GET",
    )
    config.add_route(
        name="update_internal_notes",
        pattern="capability/request/{capability_request_id}/version/{version_id}/internal_notes",
        request_method="POST",
    )
    # PUT
    config.add_route(
        name="update_file_for_latest",
        pattern="capability/request/{capability_request_id}/version/latest/files/{filename}",
        request_method="PUT",
    )
    # View the workflow metadata
    config.add_route(
        name="view_version_metadata",
        pattern="capability/request/{capability_request_id}/version/{version_id}/workflow_metadata",
        request_method="GET",
    )
    # Update the workflow metadata
    config.add_route(
        name="edit_version_metadata",
        pattern="capability/request/{capability_request_id}/version/{version_id}/workflow_metadata",
        request_method="POST",
    )
    # Get email information for a capability version
    config.add_route(
        name="get_user_email",
        pattern="capability/request/{capability_request_id}/version/{version_id}/user_email",
        request_method="GET",
    )


def capability_execution_routes(config: Configurator) -> None:
    """
    Server routes related to capability executions

    :param config: Pyramid server config object
    """

    request_url = "capability/request/execution/{wf_request_id}"

    # GET
    config.add_route(
        name="get_execution_from_workflow_id",
        pattern=f"{request_url}",
        request_method="GET",
    )
    config.add_route(
        name="get_execution_by_version",
        pattern="capability/request/{request_id}/version/{version_id}/execution",
        request_method="GET",
    )

    # POST
    config.add_route(
        name="create_version_from_previous_execution_script",
        pattern=f"{request_url}/create_version",
        request_method="POST",
    )


def casa_matrix_routes(config: Configurator) -> None:
    """
    Server routes related to the CASA version matrix.

    :param config: Pyramid server config object
    """

    # GET
    config.add_route(name="get_casa_version", pattern=f"casa_matrix/version", request_method="GET")
    config.add_route(name="get_casa_versions", pattern=f"casa_matrix/versions", request_method="GET")
    config.add_route(name="get_casa_recipe", pattern=f"casa_matrix/recipe", request_method="GET")

    # POST
    config.add_route(name="add_casa_version", pattern=f"casa_matrix/db/version", request_method="POST")
    config.add_route(name="update_casa_version", pattern=f"casa_matrix/db/version", request_method="PUT")
    config.add_route(name="delete_casa_version", pattern=f"casa_matrix/db/version", request_method="DELETE")
    config.add_route(name="make_casa_links", pattern=f"casa_matrix/links", request_method="POST")
